#include "math.h"
#include "stdlib.h"
#include "string.h"
#include "ctype.h"
#include "fpoint.h"

//
// these are used to add asm to the c code for speed without
// writing the full code in asm
//
#define umul(x,y) _asm([_BC,_DE,_HL],_BC=(unsigned)x,_DE=(unsigned)y," mul","ld e,c"," ld d,b"," ld bc,hl",_BCDE)


#define shiftl(x) _asm(_HL=(unsigned)&x," xor a","  rl (hl)","inc hl",\
	" rl (hl)","inc hl",\
	" rl (hl)","inc hl",\
	" rl (hl)")
#define shiftr1(x) _asm(_BCDE=(unsigned long)x," xor a"," rr b"," rr c"," rr de",_BCDE)

#define or_one(x) _asm(_HL=(unsigned)&x," set 0,(hl)")
#define clr_15i(x) _asm([_HL],_HL=(unsigned)&x," inc hl"," res 7,(hl)");
#define is_negi(x)_asm([_A],_HL=(unsigned)x," ld a,h"," and 0x80",_A)
#define is_neg(x) _asm([_HL,_A],_HL=(unsigned)&x," inc hl"," inc hl"," inc hl"," ld a,(hl)"," and 0x80",_A)
#define get_rem _asm([_HL,_BC,_DE]," exx",_BCDE)
#define mul10i(x) _asm([_BCDE],_BC=x," ld de,10"," mul",_BC)



#pragma offset_labels on
#pragma warn -760

near FPOINT fp_mul(FPOINT tx, FPOINT ty)
{
	fp_val rval;
	unsigned i1,i2,f1,f2;
	char neg=0;
	if (is_neg(tx))
	{
		neg=1;
		tx=-tx;
	}
	if (is_neg(ty))
	{
		++neg;
		ty=-ty;
	}
	f1=(unsigned)tx;
	//
	// the ZW mul instruction is really only 15x15 bit
	// so do bit 15 of fraction manually
	//
	if (is_negi(f1))
		rval.fpoint=shiftr1(ty);
	else
		rval.fpoint=0;

	clr_15i(f1);	// clear it for 15x15
	//
    // repeat for other fraction.
	//
	f2=(unsigned)ty;
	if (is_negi(f2))
		rval.fpoint+=shiftr1(tx);
	clr_15i(f2);
	i1=getupper(tx);
	i2=getupper(ty);
	rval.f.integer+=(unsigned)umul(i1,i2);
	rval.fpoint+=umul(i1,f2);
	rval.fpoint+=umul(i2,f1);
	ty=umul(f1,f2);

//	ty+=0x8000;
	if (is_negi(ty))
		++rval.fpoint;
	rval.fpoint+=getupper(ty);
	if (neg&1)
		return -rval.fpoint;
	return rval.fpoint;
}


#define ABS(x)  ((x) < 0 ? -(x) : (x))



long int fp_div(long int dividend,
		   long int divisor)
{
//  long int quotient;
  unsigned long int dend, dor;
  unsigned long int q; //, r;
  char shift=16;

  	dend = ABS(dividend);
  	dor  = ABS(divisor);
	q=dend/dor;
	q<<=16;
  	dend=get_rem; //r;
  	while (dend && !is_neg(dend))
  	{
  		dend<<=1;
  		--shift;
  	}
	dor>>=shift;
	q+=dend/dor;
  /* the sign of the remainder is the same as the sign of the dividend
     and the quotient is negated if the signs of the operands are
     opposite */
  if (is_neg(dividend)) {
    if (!is_neg(divisor))
      return -q;
  }
  else { /* positive dividend */
    if (is_neg(divisor))
      return -q;
  }
 	return q;
} /* signed_divide */


int fp_toa(FPOINT fp,char * s,int digits)
{
	char *s1=s;
	//							  0.5,0.05,0.005,0.0005,0.00005,0
	static const unsigned rval[]={32768,3277,328,33,3,0};
	if (digits>5)
		digits=5;
	if (fp<0)
	{
		*s++='-';
		fp=-fp;
	}
	fp+=rval[digits];	// round up by adding 0.5
	utoa(getupper(fp),s,10);
	s+=strlen(s);
	fp&=0xffffl;
	if (!fp || !digits)
		return s-s1;
	*s++='.';
	while (--digits)
	{
		fp*=10;
		*s++='0'+(char)getupper(fp);
		fp&=0xffffl;
	}
	fp*=10;
	*s++='0'+(char)getupper(fp);
    *s='\0';
	return s-s1;
}

FPOINT fp_atofp(char * s)
{
	unsigned scale=1;
	unsigned long ftemp=0;
	char neg=0;
	fp_val fval;
	fval.fpoint=0;
	while (isspace(*s) && *s)
		s++;
	if (*s=='-')
	{
		neg=1;
		s++;
	}
	while (isdigit(*s))
	{
		fval.f.integer=mul10i(fval.f.integer);
		fval.f.integer+=(int)*s++-'0';
	}
	if (!*s || *s++!='.')
	{
		if (neg)
			return -fval.fpoint;
		else
			return fval.fpoint;
	}

	//
	// have int, now do fraction.
	//
	while (isdigit(*s))
	{
		ftemp=umul(ftemp,10);
		if (getupper(ftemp))
		{
			ftemp/=10;
			break;	// too many digits
		}
		ftemp+=(int)*s++-'0';
		scale=mul10i(scale);
	}
	ftemp<<=16;
	fval.f.fract=(unsigned)(ftemp/scale);
	if (neg)
		return -fval.fpoint;
	return fval.fpoint;

}
