#ifndef _FPOINT_H
#define _FPOINT_H
/*

Fixed point Library, 2006 Scott Henion, www.shdesigns.org

Description:

This lib add an FPOINT data type. It is defined as a signed long.

The fixed-point value is formatted as a 16-bit integer and a 16-bit fraction
iiiiiiiiiiiiiii.fffffffffffffff

It has a range of -32768.99999 through 32767.99999

The code is faster and smaller than floating point.

Multiply takes about 2/3 the time of floating point.

Add/subrtact are very fast as it is treated as long values.

Divide is a bit slow as it has to do 2 32-bit divides for the integer and fraction.

Usage:

Define variables as FPOINT:

	FPOINT f1,f2,f3;	// used below

Addition:
     f3=f1+f2;	// uses long add

Subtraction:
     f3=f1-f2;	// uses long subtraction

Multiplication:
	f3=fp_mul(f1,f2);	// f3=f1*f2

Division:
	f3=fp_div(f1,f2);	// f3=f1/f2

Get intger part:
	int x=get-int(f1);	// get integer part of f1

Convert to ascii:
	char buff[12];	// may be from 1 to 11 chars
	fp_toa(f1,buff,digits);	// digits in number of fractional digits.

Convert from ascii:
	f1=fp_atofp("12345.67890");	// can be any string including '-nnn.nnn'

*/

typedef long int FPOINT;
typedef unsigned long int FPOINTU;

typedef struct { unsigned fract,integer; } fp_parts;
typedef union {
	fp_parts f;
	unsigned long fpoint;
	} fp_val;

#define FP_CONST(x) ((long int)(x*65536.0+0.000007629394))
#define FP_CONSTN(x) ((long int)(x*65536.0-0.000007629394))

// get upper word of long
#define getupper(x) _asm([_HL],_HL=(unsigned)&x," inc hl"," inc hl"," ld hl,(hl)",_HL)

#define fp_int(x) getupper(x)

near FPOINT fp_mul(FPOINT x, FPOINT y);	// multiply x*y
FPOINT fp_div(FPOINT x, FPOINT y);  // return x/y and store remainder (remainder may be null)
int fp_toa(FPOINT fp,char * s,int digits);	// format fp as text, returns length of string
FPOINT fp_atofp(char * s);

#endif
